/*
 *  Copyright (C) 2006 by Matthew Spencer
 *
 *  This program is free software; you may redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  For any other enquiries write to Matthew Spencer, School of Biological Sciences,
 *  University of Liverpool, Crown Street, Liverpool, L69 7ZB, UK.
 *
 *  Or send E-mail to m.spencer@liverpool.ac.uk
 *
 *  Uses the NAG FORTRAN library and C header files: these are not free software...
 *
 *  Uses the Mersenne Twister pseudorandom number generator.  Copyright notice from mt19937ar.c:
 *   A C-program for MT19937, with initialization improved 2002/1/26.
 *  Coded by Takuji Nishimura and Makoto Matsumoto.
 *
 *  Before using, initialize the state by using init_genrand(seed)  
 *  or init_by_array(init_key, key_length).
 *
 *  Copyright (C) 1997 - 2002, Makoto Matsumoto and Takuji Nishimura,
 *  All rights reserved.                          
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *    3. The names of its contributors may not be used to endorse or promote 
 *       products derived from this software without specific prior written 
 *       permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *  Any feedback is very welcome.
 *  http://www.math.sci.hiroshima-u.ac.jp/~m-mat/MT/emt.html
 *  email: m-mat @ math.sci.hiroshima-u.ac.jp (remove space)
 *
 * Uses a few functions from seq-gen.  Copyright notice for seq-gen:
 *  Sequence Generator - seq-gen, version 1.3.2
 *  Copyright (c)1996-2004, Andrew Rambaut & Nick Grassly
 *  Department of Zoology, University of Oxford			
 *  All rights reserved.                          
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    1. Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *
 *    2. Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *
 *    3. The names of its contributors may not be used to endorse or promote 
 *       products derived from this software without specific prior written 
 *       permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 *  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 *  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 *  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 *  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
 *  Any feedback is very welcome.
 *  http://evolve.zoo.ox.ac.uk/software/Seq-Gen/
 *  email: andrew.rambaut@zoo.ox.ac.uk
 *
 *
 *
 *LIST OF CHANGES
 * 28 September 2006: incorporated Mersenne Twister code from mt19937ar.c by Takuji Nishimura and Makoto Matsumoto
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <getopt.h>
#include <ctype.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include "array_funcs.h"
#include "nagmk21_us.h"
#include "mtrand.h"

#define PROGRAM_NAME "LVfit"
#define PROGRAM_VERSION "0.2"
#define COPYRIGHT "Copyright Matthew Spencer, University of Liverpool, 2006 (m.spencer@liverpool.ac.uk)"
#define LEN 1000
#define TRUE             1
#define FALSE            0
#define ODEMETHOD 2 /*4th-5th order Runge-Kutta*/
#define ODETOL 1e-6
#define STIFFTOL 1e-6
#define ODETHRESH 1e-8
#define PROB_PROBLEM -1e-4
#define MAXOPT 5 /*number of times we'll restart optimization if we run out of function evaluations*/
#define WARNSIZE 0.1 /*show warning if stiff and standard solver log likelihoods differ by this much or more*/
#define max( a, b ) ( ((a) > (b)) ? (a) : (b) )

typedef int boolean;
/*structure to hold data*/
typedef struct
{
	int ntimes;
	int s;
	double *tset;
	int *n0;
	int **ntrans_cat;
} transdata;

/*structure to hold settings*/
typedef struct
{
	int reps;
	boolean paramfile;
	boolean x0file;
	boolean quiet;
	boolean useoutfile;
	boolean isstiff;
	char *paramfilename;
	char *infilename;
	char *outfilename;
} parameters;

/*these are global because I couldn't figure out an easy way to pass them to the derivative and objective functions via the NAG library*/
transdata *td;/*this will hold the data*/
double *thetab;/*this will hold back-transformed parameters for ODE solution*/
double *THRES, *WORK, *YSTART, *YGOT,*YPGOT,*YMAX;/*these are all needed by the NAG ODE solver*/
double **Pset_cat;/*this will hold estimated transition probabilities*/
double *xnow;/*current frequencies*/

void ReadParams(int argc, char **argv, parameters *params);
void LV_ufun(int *nvar, double theta[], double *l, int IUSER[], double USER[]);
void init_theta(double *theta, int nvar, parameters *params);
void readparamsfromfile(double *theta, int nvar, parameters *params);
void back_LVparms(double *theta, int nvar);
void free_transdata(transdata *td);
transdata *make_transdata(int ntimes, int s);
transdata *ReadData(char *filename);
void printtd(transdata *td);
parameters *make_params();
void free_params(parameters *params);
void print_params(double *theta,int *nvar, parameters *params);
double get_xi0(int i);
double get_aij(int i,int j, int s);
int get_pijind(int i, int j, int s);
void Fderivs(double *T, double Y[], double YP[]);
boolean ode_nstate_P(double tfinal, double *xnow, int ti, int LENWRK);
boolean ode_nstate_Pstiff(double tfinal, double *xnow, int ti, int LENWRK);
void Jacobian(double *T, double Y[], double Jac[]);
double LVlikefun();
void print_thetab(double *theta, double negl, double negl2, int nvar, double *W, int IFAIL, double etime, int rep, FILE *outfile, int odestate, int nopt,parameters *params);
void print_inittheta(double *theta, int nvar, int rep, FILE *outfile);
boolean check_Pset();

int main(int argc, char **argv)
{
	int nvar,r,NEQ,LENWRK;
	double *theta;
	unsigned long randomSeed;
	parameters *params;
	int IUSER[4];/*user-supplied extra ints for objective function*/
	double USER[1];/*user-supplied extra doubles for objective function: not used*/
	int IBOUND,IFAIL=-1,LIW,LW;
	double negl=0.,negl2=0.,etime;
	int *IW;
	double *W, *BL, *BU;
	time_t tzero, tnow;
	FILE *outfile;
	int nopt,optdone,oldfail,i;
		
	/*initialize pseudorandom number generator*/
	randomSeed = CreateSeed();
	init_genrand(randomSeed);

	/*read settings*/
	params=make_params();
	ReadParams(argc,argv,params);
	
	/*read the data from a file*/
	td=ReadData(params->infilename);

	/*set up memory*/
	nvar=(td->s)*(td->s)-1;
	NEQ=td->s*td->s;
	/* if(!params->isstiff) LENWRK=32*NEQ; */
/* 	else LENWRK=(12+NEQ)*NEQ+50;/\*for stiff system*\/ */
	LENWRK=max(32*NEQ,(12+NEQ)*NEQ+50);
	WORK=make_1darray(LENWRK);
	thetab=make_1darray(nvar+1);
	theta=make_1darray(nvar);
	THRES=make_1darray(NEQ);
	YSTART=make_1darray(NEQ);
	YGOT=make_1darray(NEQ);
	YPGOT=make_1darray(NEQ);
	YMAX=make_1darray(NEQ);
	Pset_cat=make_2darray(td->s,td->ntimes * td->s);
	xnow=make_1darray(td->s);
	
	/*open the output file*/
	if(params->useoutfile){
		if((outfile=fopen(params->outfilename,"w")) == NULL)
		{
    			fprintf(stderr,"Couldn't open :%s for writing\n",params->outfilename);
			exit(EXIT_FAILURE);
  		}
	}
	else outfile=stdout;
	fprintf(outfile,"%s version %s\n%s\n\nCommand:\n",PROGRAM_NAME,PROGRAM_VERSION,COPYRIGHT);
	for(i=0;i<argc;i++) fprintf(outfile,"%s%s",argv[i],(i<argc-1) ? " " : "");/*write arguments to output file*/
	fprintf(outfile,"\n");

	/*set up optimizer*/
	IBOUND=1;/*no bounds on variables*/
	LIW=nvar+2;
	IW=(int*)calloc(LIW,sizeof(int));/*will hold info on active constraints*/
	if(IW==NULL){
		fprintf(stderr,"ERROR: calloc failed: IW");
		exit(EXIT_FAILURE);
	}
	LW=nvar*(nvar-1)/2+12*nvar;
	if(LW<13) LW=13;
	W=make_1darray(LW);/*will hold gradient approximation and info on condition of Hessian*/
	BL=make_1darray(nvar);/*lower bounds, not used*/
	BU=make_1darray(nvar);/*upper bounds, not used*/
	if(params->paramfile==TRUE) init_theta(theta,nvar,params);

	if(params->quiet==FALSE){
		print_params(theta,&nvar,params);
		IUSER[0]=1;/*LV_ufun will show progress*/	
	}
	else IUSER[0]=0;
	IUSER[1]=LENWRK;/*size of work array for ODE solver*/
	IUSER[2]=params->isstiff;/*which solver to use*/

	if(params->reps==0){/*evaluate likelihood for these parameters*/
	  if(params->paramfile==FALSE) init_theta(theta,nvar,params);/*construct parameter theta: random initial rates, initial frequency estimates*/
		tzero=time(NULL);
		IUSER[3]=0;/*will get set to -1 if ODE solution fails*/
		LV_ufun(&nvar,theta,&negl,IUSER,USER);
		oldfail=IUSER[3];
		IUSER[3]=0;
		IUSER[2]=(!IUSER[2]);/*use the other solver*/
		LV_ufun(&nvar,theta,&negl2,IUSER,USER);
		IUSER[2]=(!IUSER[2]);/*set back to original*/
		IUSER[3]=oldfail;
		tnow=time(NULL);
		etime=difftime(tnow,tzero);
		print_thetab(theta,negl,negl2,nvar,W,IFAIL,etime,0,outfile,IUSER[3],0,params);
	}
	else for(r=0;r<params->reps;r++){/*run optimization*/
		tzero=time(NULL);
		if(params->paramfile==FALSE) init_theta(theta,nvar,params);/*construct parameter theta: random initial rates, initial frequency estimates*/
		print_inittheta(theta,nvar,r+1,outfile);/*print out initial estimates*/
			nopt=0;
			optdone=0;
			IUSER[3]=0;/*will get set to -1 if ODE solution fails*/
			while(!optdone){
				IFAIL=-1;/*noisy exit if optimization didn't work*/

				e04jyf_(&nvar,&IBOUND,LV_ufun,BL,BU,theta,&negl,IW,&LIW,W,&LW,IUSER,USER,&IFAIL);/*call optimizer with parameter theta*/
				if(IFAIL==2){/*ran out of function evals*/
					nopt++;/*count as completed loop because we ran out of evaluations*/
					optdone=(nopt>=MAXOPT);/*give up if we used too many attempts*/
				}
				else{
					optdone=1;/*either succeeded, or failed for some reason other than running out of function evaluations*/
					if(IFAIL==0) nopt++;/*count as completed loop if successful*/
				}
			}
		oldfail=IUSER[3];
		IUSER[3]=0;
		IUSER[2]=(!IUSER[2]);/*use the other solver*/
		LV_ufun(&nvar,theta,&negl2,IUSER,USER);
		IUSER[2]=(!IUSER[2]);/*set back to original*/
		IUSER[3]=oldfail;
		tnow=time(NULL);
		etime=difftime(tnow,tzero);
		print_thetab(theta,negl,negl2,nvar,W,IFAIL,etime,r+1,outfile,IUSER[3],nopt,params);
		sleep(1);/*make sure we get a different set of pseudorandom numbers*/
	}
	
	if(params->useoutfile) fclose(outfile);
	/*free memory*/
	free_2darray(Pset_cat,td->s);
	free(xnow);
	free_transdata(td);
	free(thetab);
	free(theta);
	free_params(params);
	free(THRES);
	free(WORK);
	free(YSTART);
	free(YGOT);
	free(YPGOT);
	free(YMAX);
	free(IW);
	free(W);
	free(BL);
	free(BU);
	return EXIT_SUCCESS;
}



/*read parameters*/
void ReadParams(int argc, char **argv, parameters *params)
{
	int c;
	extern char *optarg;

	opterr = 0;
	while ((c = getopt (argc, argv, "sqnxp:r:h")) != -1) switch (c) {
		case 'h':/*help message*/
			printf("\n"); 
			printf(PROGRAM_NAME);
			printf(" ");
			printf(PROGRAM_VERSION);
			printf(" Copyright (C) 2006 Matthew Spencer, School of Biological Sciences, University of Liverpool (m.spencer@liverpool.ac.uk).\n");
			printf("Usage: LVfit [-h -q -n -xp paramfile -r #reps -s] infile outfile\n");
			printf("Options:\n");
			printf("infile: the data file\n");
			printf("outfile: the output file (if not specified, goes to stdout)\n"); 
			printf("-h: This help message\n");
			printf("-q: Quiet operation (default)\n");
			printf("-n: Noisy operation\n");
			printf("-p paramfile: read initial rate parameters from a file (otherwise generate at random)\n");
			printf("-x: read initial frequencies from the rate parameter file (otherwise use add-one pseudocount estimates)\n");
			printf("NOTE: rates are specified as a row of text, and run across then down the rate matrix, excluding the diagonal\n");
			printf("If initial frequencies are read from the file, they come before the rates\n");
			printf("-r #: number of replicate optimizations to run (default is 1)\n");
			printf("with -r0, evaluates likelihood for current parameters but doesn't optimize\n");
			printf("-s: use stiff solver (otherwise use a standard ODE solver)\n");
			printf("\n");
			printf("Input format:\n");
			printf("The data input file should be ASCII text.  The first line gives the number of time intervals.\n");
			printf("The second line gives the number of states.\n");
			printf("The third line is the ends of the time intervals at which samples were taken, separated by spaces.\n");
			printf("It is assumed that the first sample was taken at time 0, which is not included.\n");
			printf("The remaining lines give the counts of observed transitions between source states (columns) and destination states (rows)\n");
			printf("for all time intervals.  The matrices of transition counts are concatenated as a row F1 F2 F3..., where Fi is the counts of transitions\n");
			printf("for time interval i.  Entries are separated by spaces.\n");
			printf("Note that the last state is assumed to be empty space.\n");
			printf("If initial parameter estimates are supplied as a file (-p), this should be a separate ASCII text file.  The parameters are separated by spaces\n");
			printf("and are supplied as initial frequencies (only if -xp) followed by the rates, moving across then down the rate matrix, excluding the diagonal.\n");
			printf("\n");
			printf("Output format:\n");
			printf("For each replicate optimization:\n");
			printf("The starting initial frequencies and rates.\n");
			printf("IFAIL: the condition of the optimizer on exit.\n");
			printf("The number of optimization loops run (if the maximum number of function evaluations in the NAG library optimizer is exceeded,\n");
			printf("the optimization is restarted from the current estimates up to four times).\n");
			printf("If IFAIL=3 and 0 optimization loops were completed, there may be a problem calculating the likelihood (see notes).\n");
			printf("If PROBLEMS WITH ODE SOLUTION is reported, this means that the ODE solver failed because of numerical problems,\n");
			printf("even though the current point may not be a minimum.\n");
			printf("The next lines report the log likelihood, estimated initial frequencies and interaction rates.\n");
			printf("The log likelihood is evaluated using both the requested ODE solver and the alternative one. If these results differ by more than %f, a warning is printed.\n",WARNSIZE);
			printf("Next, the gradient of the log likelihood at the current estimates (parameters ordered as in an input parameter file, see input format).\n");
			printf("The estimated condition number of the Hessian is reported: very large values may indicate that the parameters are not identifiable,\n");
			printf("but if the optimization failed, the condition number estimate may not be reliable.\n");
			printf("Finally, the clock time in seconds used for the optimization is reported.\n");
			printf("\n");
			printf("Notes\n");
			printf("Situations in which the results may be unreliable: if both IFAIL=3 (the point is not a minimum but a better one could not be found)\n");
			printf("and 0 optimization loops were completed, there may be a problem calculating the likelihood.\n");
			printf("If the stiff and non-stiff solvers give very different answers, this is likely to be the case.\n");
			printf("\nThere are no checks on whether the input data format is correct.  It's a good idea to run with -n first of all,\n");
			printf("which will display the data as read.\n");
			exit(EXIT_SUCCESS);
		break;
		case 'p':/*read initial parameters from file*/
			params->paramfile=TRUE;
			strcpy(params->paramfilename,optarg);
		break;
		case 'x':/*read initial frequencies from the rate parameter file*/
			params->x0file=TRUE;
		break;
		case 'q':/*quiet operation*/
			params->quiet=TRUE;
		break;
		case 'n':/*noisy operation*/
			params->quiet=FALSE;
		break;
		case 'r':/*number of replicates*/
			params->reps=atoi(optarg);
		break;
		case 's':/*use stiff solver*/
			params->isstiff=TRUE;
		break;
		case '?':
		if (isprint (optopt))
			fprintf (stderr, "Unknown option `-%c'.\n", optopt);
		else
			fprintf (stderr,"Unknown option character `\\x%x'.\n",optopt);
			exit(EXIT_FAILURE);
		default:
			abort ();
	}
    

	if(argc > optind){
		sprintf(params->infilename,"%s",argv[optind]);
    	}
	else{/* ask for the input-file */
		printf("\nEnter input file : ");
		scanf("%s",params->infilename);
	}
	if(argc > optind+1){
		sprintf(params->outfilename,"%s",argv[optind+1]);
		params->useoutfile=TRUE;
    	}
	else params->useoutfile=FALSE;

}

/*the LV objective function for minimization*/
/*nvar is number of parameters
theta is their current values
l will hold the negative log likelihood
IUSER and USER can supply user-specified integer and double things needed to evaluate the likelihood:
IUSER[0]=1 indicates noisy operation
ISUER[1] is size of work array for ODE solver
IUSER[2] is TRUE if we should use a stiff solver
IUSER[3] is initially 0, but gets changed to -1 if we encounter ODE solution problems (from then on the likelihood
will not be updated and the optimizer will quit soon after)*/
void LV_ufun(int *nvar, double theta[], double *l, int IUSER[], double USER[])
{
	double tzero;
	int i,j;
	boolean ODEsuccess=TRUE;
	
	/*TEST***************/
	/*double *Jac;
	double ttest;*/
	/********************/
	
	if(IUSER[3]==0){/*no ODE problems yet: if there are problems we stop updating the likelihood*/
		back_LVparms(theta,*nvar);/*back-transform parameters*/

		/*initialize at time zero*/
		tzero=0.;
		for(i=0;i<td->s;i++){
			xnow[i]=get_xi0(i);
			for(j=0;j<td->s*td->ntimes;j++) Pset_cat[i][j]=0.;
		}
		/*for each time step*/
		for(i=0;i<td->ntimes;i++){
			if (IUSER[2]==FALSE){
				if(!ode_nstate_P(td->tset[i]-tzero,xnow,i,IUSER[1])) ODEsuccess=FALSE;/*solve the ODEs and construct the P matrix*/
			}
			else if(!ode_nstate_Pstiff(td->tset[i]-tzero,xnow,i,IUSER[1])) ODEsuccess=FALSE;/*solve the ODEs and construct the P matrix using stiff solver*/
			tzero=td->tset[i];
		}
		if(ODEsuccess){
			if(check_Pset()==TRUE){/*set any small negative probabilities (due to solution error) to zero*/
				*l=-LVlikefun();/*calculate the negative log likelihood*/
				if(IUSER[0]==1) printf("%f \n",*l);/*current likelihood*/
				if (isinf(*l)||isnan(*l)){/*stop the optimizer*/
					fprintf(stderr,"likelihood %f, abandoning optimization\n",*l);
					IUSER[3]=-1;/*stop updating negl*/
				}
			}
			else IUSER[3]=-1;/*there were some large negative probabilities: stop optimizer*/
		}
		else IUSER[3]=-1;/*don't update negl so the optimizer will stop soon*/
	}
	
	/*TEST THE JACOBIAN*/
	/*Jac=make_1darray(td->s*td->s*td->s*td->s);
	for(i=0;i<td->s*td->s*td->s*td->s;i++) Jac[i]=-999.999;
	ttest=1.0;
	Jacobian(&ttest,YSTART,Jac);
	free(Jac);*/
	/******************/
}

/*initial parameter guesses*/
void init_theta(double *theta, int nvar, parameters *params)
{
	int i;
	double ns;

	/*initial frequencies*/
	ns=td->n0[td->s-1]+1.;
	if(params->x0file==FALSE)for(i=0;i<td->s-1;i++) theta[i]=log((td->n0[i]+1.)/ns);/*add-one pseudocounts for initial frequencies*/
/*	if(params->paramfile==FALSE) for(i=td->s-1;i<nvar;i++) theta[i]=genrand_real1();*//*random initial rates: log rates uniform on [0,1] as in matlab*/
	if(params->paramfile==FALSE) for(i=td->s-1;i<nvar;i++) theta[i]=log(genrand_real1());/*random initial rates: untransformed rates uniform on [0,1]*/
	else readparamsfromfile(theta, nvar, params);/*read from file*/
	back_LVparms(theta,nvar);/*back-transform parameters*/
}

/*back-transform the parameters*/
void back_LVparms(double *theta, int nvar)
{
	int i;
	double psum;
	
	psum=1.;
	for(i=0;i<td->s-1;i++) psum+=exp(theta[i]);
	for(i=0;i<td->s-1;i++) thetab[i]=exp(theta[i])/psum;
	thetab[td->s-1]=1./psum;
	for(i=td->s;i<nvar+1;i++) thetab[i]=exp(theta[i-1]);
}

/*allocate memory for a structure containing data*/
transdata *make_transdata(int ntimes, int s)
{
	transdata *td;
	int i;
	
	td=(transdata *)calloc(1,sizeof(transdata));
	if(td==NULL){
		fprintf(stderr,"ERROR: calloc failed: transdata");
		exit(EXIT_FAILURE);
	}
	td->ntimes=ntimes;
	td->s=s;
	td->ntrans_cat=(int **)calloc(s,sizeof(int*));
	if(td->ntrans_cat==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: td->ntrans_cat");
		(void)exit(EXIT_FAILURE);
	}

	for(i=0;i<s;i++){
		td->ntrans_cat[i]=(int*)calloc(s*ntimes,sizeof(int));
		if(td->ntrans_cat[i]==NULL){
			fprintf(stderr,"ERROR: calloc failed: td->ntrans_cat");
			exit(EXIT_FAILURE);
		}
	}
	td->tset=make_1darray(ntimes);
	td->n0=(int *)calloc(s,sizeof(int));
	if(td->n0==NULL){
		fprintf(stderr,"ERROR: calloc failed: td->n0");
		exit(EXIT_FAILURE);
	}
	return td;
}

/*free memory for structure containing data*/
void free_transdata(transdata *td)
{
	int i;
	for(i=0;i<td->s;i++) free(td->ntrans_cat[i]);
	free(td->ntrans_cat);
	free(td->tset);
	free(td->n0);
	free(td);
}

/*read data from an ASCII file*/
/*first now is ntimes (excludign zero)
second row is s (number of states)
then set of times (excluding zero)
then a block of concatenated transition counts*/
transdata *ReadData (char *filename)
{
	FILE *infile;
	char *myline;
	int ntimes,s,i,j;
	transdata *td;

	/*memory for a line*/
	myline=(char*)calloc(LEN,sizeof(char));
	if(myline==NULL){
		fprintf(stderr,"ERROR: calloc failed: ReadData");
		exit(EXIT_FAILURE);
	}

	/*open the data file*/
	if( (infile=fopen(filename,"r")) == NULL)
	{
    		fprintf(stderr,"File not found:%s\n",filename);
		exit(EXIT_FAILURE);
  	}

	fscanf(infile,"%d\n",&ntimes);
	fscanf(infile,"%d\n",&s);

	/*get memory for the data*/
	td=make_transdata(ntimes, s);

	/*read tset*/
	for(i=0;i<ntimes;i++) fscanf(infile,"%lf",&td->tset[i]);

	/*read ntrans_cat*/
	for(i=0;i<s;i++){
		for(j=0;j<ntimes*s;j++) fscanf(infile,"%d",&td->ntrans_cat[i][j]);
		fscanf(infile,"\n");
	}

	/*calculate n0*/
	for(j=0;j<s;j++){
		td->n0[j]=0;
		for(i=0;i<s;i++){
			td->n0[j]+=td->ntrans_cat[i][j];
		}
	}

	/*close the data file*/
	fclose(infile);
	free(myline);
	return td;
}


/*print out the data*/
void printtd(transdata *td)
{
	int i,j;

	printf("%d times, %d states\n",td->ntimes,td->s);
	printf("Sample times: ");
	for(i=0;i<td->ntimes;i++) printf("%f ",td->tset[i]);
	printf("\n");
	printf("Transition counts:\n");
	for(i=0;i<td->s;i++){
		for(j=0;j<(td->s * td->ntimes);j++){
			printf("%d ",td->ntrans_cat[i][j]);
		}
		printf("\n");
	}
	printf("Initial counts:\n");
	for(i=0;i<td->s;i++){
		printf("%d\n",td->n0[i]);
	}
	printf("\n\n");
}

/*allocate memory for parameter structure*/
parameters *make_params()
{
	parameters *params;
	params=(parameters *)calloc(1,sizeof(parameters));
	if(params==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: params");
		(void)exit(EXIT_FAILURE);
	}
	params->infilename=(char *)calloc(LEN,sizeof(char));	
	if(params->infilename==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: params->infilename");
		(void)exit(EXIT_FAILURE);
	}
	params->paramfilename=(char *)calloc(LEN,sizeof(char));	
	if(params->paramfilename==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: params->paramfilename");
		(void)exit(EXIT_FAILURE);
	}
	params->outfilename=(char *)calloc(LEN,sizeof(char));	
	if(params->outfilename==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed: params->outfilename");
		(void)exit(EXIT_FAILURE);
	}
	params->reps=1;
	params->paramfile=FALSE;
	params->useoutfile=FALSE;
	params->x0file=FALSE;
	params->quiet=TRUE;
	params->isstiff=FALSE;
	return params;
}

/*free parameter structure memory*/
void free_params(parameters *params)
{
	free(params->infilename);
	free(params->paramfilename);
	free(params->outfilename);
	free(params);
}

/*print out current parameter settings*/
void print_params(double *theta, int *nvar, parameters *params)
{
	int i,j;
	printf("Data input file: %s\n",params->infilename);
	printf("\nData:\n");
	printtd(td);/*probably won't want to keep this here once testing is done*/
	
	if(params->paramfile==TRUE){
		printf("Initial parameter input file: %s\n",params->paramfilename);
		back_LVparms(theta,*nvar);/*back-transform parameters*/
		if(params->x0file==TRUE){
			printf("Initial frequencies from file: ");
			/*print the frequencies*/
			for(i=0;i<td->s;i++) printf("%.16f ",thetab[i]);
			printf("\n");	
		}
		else printf("Initial frequencies: add-one pseudocounts\n");
		/*print the rates*/
		printf("Rates:\n");
		for(i=0;i<td->s;i++){
			for(j=0;j<td->s;j++){
				if(i==j) printf("%.16f ",0.);
				else printf("%.16f ",get_aij(i,j,td->s));
			}
			printf("\n");
		}
	}
	else printf("Initial rates generated at random (and initial frequencies from add-one pseudocounts)\n");
	printf("%d replicate optimizations\n",params->reps);
	if(params->useoutfile==TRUE) printf("Output to %s\n",params->outfilename);
	if(params->quiet==TRUE) printf("Quiet operation\n");
	else printf("Noisy operation\n");
	if(params->isstiff==TRUE) printf("Using stiff solver\n");
	else printf("Using standard solver\n");
	printf("\n");
}

/*read initial rate estimates from a file*/
void readparamsfromfile(double *theta, int nvar, parameters *params)
{
	int i;
	double xe;
	FILE *pfile;
	/*open the data file*/
	if((pfile=fopen(params->paramfilename,"r")) == NULL)
	{
    		fprintf(stderr,"File not found:%s\n",params->paramfilename);
		exit(EXIT_FAILURE);
  	}
	if(params->x0file==TRUE){/*read initial frequencies from file*/
		for(i=0;i<td->s-1;i++){
			fscanf(pfile,"%lf",&theta[i]);
		}
		fscanf(pfile,"%lf",&xe);/*the frequency of the last state*/
		for(i=0;i<td->s-1;i++) theta[i]=log(theta[i]/xe);/*transformation to indepedent parameters on real line*/
	}
	for(i=td->s-1;i<nvar;i++){
		fscanf(pfile,"%lf",&theta[i]);
		theta[i]=log(theta[i]);/*we'll work with log values in optimization*/
	}
	fclose(pfile);
}

/*get the ith initial frequency*/
double get_xi0(int i)
{
	return thetab[i];
}

/*get the ijth interaction rate*/
/*this assumes the parameters in the vector go across then down the rate matrix, skipping the diagonals
This is the opposite to what matlab does*/
double get_aij(int i,int j, int s)
{
	return thetab[i*(s-1)+j-(j>i)+s];
}

/*which equation is p_ij?*/
int get_pijind(int i, int j, int s)
{
	return (j+1)*s+i;
}


/*define the differential equations*/
/*T is time, Y is current values of variables, YP will contain derivatives*/
void Fderivs(double *T, double Y[], double YP[])
{
	int i,j,k,c,e;
	
	e=td->s-1;/*empty space state*/
	for(i=0;i<e;i++){/*the x_i equations, i \neq e*/
		YP[i]=-get_aij(e,i,td->s)*Y[i];/*transitions into empty space: -a_ei * x_i*/
		for(k=0;k<td->s;k++){
			if(!(k==i)){
				YP[i]+=get_aij(i,k,td->s)*Y[i]*Y[k];/*transitions into state i from k: a_ik * x_i * x_k*/
				if(!(k==e)) YP[i]-=get_aij(k,i,td->s)*Y[k]*Y[i];/*transitions out of state i into k: -a_ki * x_k * x_i*/
			}
		}
	}

	/*x_e equation*/
	YP[e]=0;
	for(k=0;k<e;k++){
		YP[e]+=get_aij(e,k,td->s)*Y[k];/*transitions into empty space: +a_ek * x_k */
		YP[e]-=get_aij(k,e,td->s)*Y[k]*Y[e];/*transitions out of empty space: -a_ke * x_k * x_e */
	}

	c=td->s;
	for(j=0;j<e;j++){/*the p_ij equations, skipping last col: we substitute p_ij for x_i*/
		for(i=0;i<td->s;i++){
			if(i==e){
				YP[c]=0;
				for(k=0;k<e;k++){
					YP[c]+=get_aij(e,k,td->s)*Y[get_pijind(k,j,td->s)];
					YP[c]-=get_aij(k,e,td->s)*Y[k]*Y[get_pijind(e,j,td->s)];
				}
			}
			else{
				YP[c]=-get_aij(e,i,td->s)*Y[get_pijind(i,j,td->s)];
				for(k=0;k<td->s;k++){
					if(!(k==i)){
						YP[c]+=get_aij(i,k,td->s)*Y[get_pijind(k,j,td->s)]*Y[i];
						if(!(k==e)) YP[c]-=get_aij(k,i,td->s)*Y[k]*Y[get_pijind(i,j,td->s)];
					}
				}
			}
			c=c+1;/*keep track of which equation we're on*/
		}
	}
}

/*build the transition probability matrix for a single time interval*/
/*tfinal is the time interval, xnow is vector of initial conditions, ti is which time interval we're on*/
boolean ode_nstate_P(double tfinal, double *xnow, int ti, int LENWRK)
{
	double ZERO=0.;
	double ONE=1.;
	int IFAIL,NEQ,METHOD,ERRASS;
	double HSTART,TGOT,TSTART,TOL;
	int i,j,r,c;
	double psum;
	
	/*things needed for the NAG library*/
	NEQ=td->s*td->s;
	METHOD=ODEMETHOD;
	TOL=ODETOL;
	TSTART=ZERO;
	ERRASS=0;
	HSTART=ZERO;
	for(i=0;i<NEQ;i++) THRES[i] = ODETHRESH;
	for(i=0;i<td->s;i++) YSTART[i]=xnow[i];/*current state frequencies*/
	j=0;
	for(i=td->s;i<NEQ;i++){/*initialize transition probs*/
		if(j==0) YSTART[i]=ONE;
		else YSTART[i]=ZERO;
		j++;
		if(j==td->s+1) j=0;
	}
	IFAIL=-1;/*used to be set to zero, but this terminates execution*/
	d02pvf_(&NEQ,&TSTART,YSTART,&tfinal,&TOL,THRES,&METHOD,"U",&ERRASS,&HSTART,WORK,&LENWRK,&IFAIL,1);/*set up the solver*/
	if(IFAIL!=0) return FALSE;
	IFAIL = -1;
	d02pcf_(Fderivs,&tfinal,&TGOT,YGOT,YPGOT,YMAX,WORK,&IFAIL);/*solve the ODEs*/
	if(IFAIL==0){/*it probably worked*/
		r=0;/*row index in Pset_cat*/
		c=ti*td->s;/*starting column index in Pset_cat*/
		for(i=td->s;i<NEQ;i++){
			Pset_cat[r][c]=YGOT[i];
			r=r+1;
			if(r==td->s){/*we reached the end of a column*/
				r=0;
				c=c+1;
			}
		}
		/*calculate the last col of the P matrix*/
		for(i=0;i<td->s;i++){
			psum=0.;
			for(j=0;j<td->s;j++){
				c=ti*td->s+j;
				psum+=Pset_cat[i][c]*xnow[j];
			}
			Pset_cat[i][(ti+1)*td->s-1]=(YGOT[i]-psum)/xnow[td->s-1];
		}
		for(i=0;i<td->s;i++) xnow[i]=YGOT[i];/*update the current frequencies*/
		return TRUE;
	}
	else return FALSE;
}

/*build the transition probability matrix for a single time interval if the system is still*/
/*tfinal is the time interval, xnow is vector of initial conditions, ti is which time interval we're on
LENWRK is the size of the work array needed*/
boolean ode_nstate_Pstiff(double tfinal, double *xnow, int ti, int LENWRK)
{
	double ZERO=0.;
	double ONE=1.;
	int IFAIL,NEQ;
	double TSTART,TOL;
	int i,j,r,c;
	double psum;

	/*things needed for the NAG library*/
	NEQ=td->s*td->s;
	TOL=STIFFTOL;
	TSTART=ZERO;
	for(i=0;i<td->s;i++) YSTART[i]=xnow[i];/*current state frequencies*/
	j=0;
	for(i=td->s;i<NEQ;i++){/*initialize transition probs*/
		if(j==0) YSTART[i]=ONE;
		else YSTART[i]=ZERO;
		j++;
		if(j==td->s+1) j=0;
	}
	IFAIL=-1;/*report error but don't exit the program if anything goes wrong*/
	/*d02ejf_(&TSTART,&tfinal,&NEQ,YSTART,Fderivs,d02ejy_,&TOL,"D",d02ejx_,d02ejw_,WORK,&LENWRK,&IFAIL,1);*//*call the stiff solver with numerical Jacobian*/
	d02ejf_(&TSTART,&tfinal,&NEQ,YSTART,Fderivs,Jacobian,&TOL,"D",d02ejx_,d02ejw_,WORK,&LENWRK,&IFAIL,1);/*call the stiff solver with analytical Jacobian*/
	if(IFAIL==0){/*it probably worked*/
		r=0;/*row index in Pset_cat*/
		c=ti*td->s;/*starting column index in Pset_cat*/
		for(i=td->s;i<NEQ;i++){
			Pset_cat[r][c]=YSTART[i];/*on exit from d02ejf_ this should be the solution*/
			r=r+1;
			if(r==td->s){/*we reached the end of a column*/
				r=0;
				c=c+1;
			}
		}
		/*calculate the last col of the P matrix*/
		for(i=0;i<td->s;i++){
			psum=0.;
			for(j=0;j<td->s;j++){
				c=ti*td->s+j;
				psum+=Pset_cat[i][c]*xnow[j];
			}
			Pset_cat[i][(ti+1)*td->s-1]=(YSTART[i]-psum)/xnow[td->s-1];
		}
		for(i=0;i<td->s;i++) xnow[i]=YSTART[i];/*update the current frequencies*/
		return TRUE;
	}
	else return FALSE;
}

/*Jacobian matrix for stiff ODE solution
T is current time (not used)
Y is current values of state variables
Jac is a square matrix of partial derivatives \delta f_i / \delta y_j
where f_i is the ith differential equation and y_j is the jth state variable
Jacobian entries accessed via a macro because C has to pass it as a one-d array
The macro takes care of the transposition too
*/
void Jacobian(double *T, double Y[], double Jac[])
{
	int b,c,e,i,j,ii,jj,k,s,NEQ;
	double ZERO=0.;
	/*TEST************/
	/*FILE *Jacfile;*/
	/****************/
		
	#define J(i,j) Jac[(i) + (td->s * td->s)*(j)] /*macro accesses entry J[i][j]*/
	s=td->s;
	e=s-1;
	NEQ=s*s;
	
	for(i=0;i<s;i++){
		for(j=0;j<s;j++){/*dx_i/dt wrt x_j*/
			if(i!=e){
				if(j==e) J(i,j)=get_aij(i,j,s)*Y[i];/*case 2: i \neq e, j=e CHECKED*/
				else if(j==i){/*case 3: i \neq e, j = i CHECKED*/
					J(i,j)=-get_aij(e,i,s);
					for(k=0;k<s;k++){
						if(k!=i){
							J(i,j)+=get_aij(i,k,s)*Y[k];
							if(k!=e) J(i,j)-=get_aij(k,i,s)*Y[k];
						}
					}
				}
				else J(i,j)=(get_aij(i,j,s)-get_aij(j,i,s))*Y[i];/*case 1: i \neq e, j \neq i, j\neq e CHECKED*/
			}
			else{
				if(j!=e) J(i,j)=get_aij(e,j,s)-get_aij(j,e,s)*Y[i];/*case 4: i=e,j \neq e CHECKED*/
				else{/*case 5: i=e,j=e CHECKED*/
					J(i,j)=ZERO;
					for(k=0;k<e;k++) J(i,j)-=get_aij(k,e,s)*Y[k];
				}
			}
		}
		for(j=s;j<NEQ;j++) J(i,j)=ZERO;/*case 6: dx_i/dt wrt p_ij: these are all zero CHECKED*/
	}
	/*now the pij entries*/
	for(j=0;j<e;j++){/*the p_ij equations, skipping last col: we substitute p_ij for x_i*/
		for(i=0;i<s;i++){
			b=get_pijind(i,j,s);/*which entry in Y is p_ij?*/
			/*now take derivatives wrt each of the other state variables c*/
			if(i!=e){
				for(c=0;c<e;c++){
					if(c!=i) J(b,c)=-get_aij(c,i,s)*Y[b];/*case 7: wrt x_c, i \neq e, c \neq i CHECKED*/
					else{
						J(b,c)=ZERO;
						for(k=0;k<s;k++)
							if(k!=i) J(b,c)+=get_aij(i,k,s)*Y[get_pijind(k,j,s)];/*case 7b: wrt x_c, i \neq e, c=i CHECKED*/
					}
				}
				J(b,e)=ZERO;/*case 8: wrt x_e, i \neq e CHECKED*/
				for(jj=0;jj<e;jj++)
					for(ii=0;ii<s;ii++){
						c=get_pijind(ii,jj,s);
						if(c==b){
							J(b,c)=-get_aij(e,i,s);/*case 9: wrt p_ij, i \neq e CHECKED*/
							for(k=0;k<e;k++){
								if(k!=i) J(b,c)-=get_aij(k,i,s)*Y[k];
							}
						}
						else if(jj==j) J(b,c)=get_aij(i,ii,s)*Y[i];/*case 10: wrt p_kj, k \neq i, i \neq e CHECKED*/
						else J(b,c)=ZERO;/*case 10b: wrt p_kl, k \neq i, l \neq j CHECKED*/
					}
			}
			else{
				J(b,e)=ZERO;/*case 11b: wrt x_e, i = e*/
				J(b,b)=ZERO;
				for(k=0;k<e;k++){
					J(b,k)=-get_aij(k,e,s)*Y[get_pijind(e,j,s)];/*case 11: wrt x_c, i = e CHECKED */
					J(b,b)-=get_aij(k,e,s)*Y[k]; /*case 12: wrt p_ij, i = e CHECKED*/
					c=get_pijind(k,j,s);
					J(b,c)=get_aij(e,k,s);/*case 13: wrt p_kj, k \neq i, i = e CHECKED*/
					for(jj=0;jj<e;jj++) if(jj!=j){
						c=get_pijind(k,jj,s);
						J(b,c)=ZERO;/*case 14: wrt p_kl, k \neq i, i = e, l \neq j CHECKED*/ 
						
					}
					c=get_pijind(e,k,s);
					if(c!=b) J(b,c)=ZERO;/*case 15: wrt p_el, l \neq j CHECKED*/
				}
			}
		}
	}	
	/*TEST: PRINT OUT THE JACOBIAN*/
	/* Jacfile=fopen("testJac.txt","w"); */
/* 	for(i=0;i<NEQ;i++) fprintf(Jacfile,"%.16f ",Y[i]); */
/* 	fprintf(Jacfile,"\n\n"); */
/* 	for(i=0;i<NEQ;i++){ */
/* 		for(j=0;j<NEQ;j++) fprintf(Jacfile,"%.16f ",J(i,j)); */
/* 		fprintf(Jacfile,"\n"); */
/* 	} */
/* 	fclose(Jacfile); */
	/**********************************/
}

/*calculate the log likelihood*/
/*no arguments because all the necessary things are global*/
double LVlikefun()
{
	int i,j;
	double l;
	
	l=0.;
	for(i=0;i<td->s;i++){
		if(thetab[i]!=0.) l+=td->n0[i]*log(thetab[i]);
		for(j=0;j<td->s*td->ntimes;j++) if(Pset_cat[i][j]!=0.) l+=td->ntrans_cat[i][j]*log(Pset_cat[i][j]);
	}
 	return l;
}

/*print the current parameter estimates thetab from transformed estimates theta and log likelihood negl
negl2 is negative log likelihood found with the other solver (stiff if we specified non-stiff and vice versa.
W is an array with values returned by NAG function e04jyf_
Entries 0 to nvar-1 are finite-difference gradient.  Entry nvar is an estimate of
the condition number of the Hessian
IFAIL indicates whether the optimization was successful
etime is clock time in seconds used for optimization
rep is replicate
outfile is destination for printing
odestate is zero if no ODE solution problems, -1 otherwise
nopt is number of times we restarted the optimization because we ran out of function evaluations
params is the input arguments we specified*/
void print_thetab(double *theta, double negl, double negl2, int nvar, double *W, int IFAIL, double etime, int rep, FILE *outfile, int odestate, int nopt,parameters *params)
{
	int i,j;

	fprintf(outfile,"\n");
	if(rep>0){
		fprintf(outfile,"IFAIL=%d: ",IFAIL);
		switch(IFAIL){/*cases from NAG documentation for E04JYF*/
			case 0:
				fprintf(outfile,"success\n");
			break;
			case 1:
				fprintf(outfile,"problem not set up correctly\n");
			break;
			case 2:
				fprintf(outfile,"max function evaluations exceeded: could restart from these conditions\n");
			break;
			case 3:
				fprintf(outfile,"not all conditions for a minimum are satisfied but a lower point could not be found\n");
			break;
			case 4:
				fprintf(outfile,"overflow\n");
			break;
			case 5: case 6: case 7: case 8:
				fprintf(outfile,"point may not be a minimum (IFAIL in range 5-8, 5 indicates the point may be quite good, 8 indicates it's unlikely to be good.  Could try a different starting point\n");
			break;
			case 9:
				fprintf(outfile,"one of the parameters got too large: could rescale the problem?\n");
			break;
			case 10:
				fprintf(outfile,"problem with forward-difference intervals: could try another starting point\n");
			break;
			default:
				fprintf(outfile,"unknown condition\n");
		}
	}
	if(params->reps>0) fprintf(outfile,"%d optimization loops were run\n",nopt);
	if(odestate==-1)fprintf(outfile,"PROBLEMS WITH ODE SOLUTION\n");
	if (params->isstiff){
	  fprintf(outfile,"-log likelihood from requested stiff solver=%.8e \n",negl);
	  fprintf(outfile,"-log likelihood from standard solver=%.8e \n",negl2);
	}
	else{
	  fprintf(outfile,"-log likelihood from requested standard solver=%.8e\n",negl);
	  fprintf(outfile,"-log likelihood from stiff solver=%.8e\n",negl2);
	}
	if(abs(negl2-negl)>WARNSIZE)
	  fprintf(outfile,"Warning: standard and stiff solver results differ by %f. Solution may be unreliable.\n",negl2-negl);
	back_LVparms(theta,nvar);/*back-transform parameters*/
	if(rep>0) fprintf(outfile,"Estimated initial frequencies:\n");
	else fprintf(outfile,"For initial frequencies:\n");
	for(i=0;i<td->s;i++) fprintf(outfile,"%.16e ",thetab[i]);
	fprintf(outfile,"\n");	
	if(rep>0) fprintf(outfile,"Estimated rates:\n");
	else fprintf(outfile,"and rates:\n");
	for(i=0;i<td->s;i++){
		for(j=0;j<td->s;j++){
			if(i==j) fprintf(outfile,"%.16e ",0.);
			else fprintf(outfile,"%.16e ",get_aij(i,j,td->s));
		}
		fprintf(outfile,"\n");
	}
	if(rep>0){
		fprintf(outfile,"\nEstimated gradient:\n");
		for(i=0;i<nvar;i++) fprintf(outfile,"%.16e ",W[i]);
		fprintf(outfile,"\nEstimated condition number of Hessian: %f\n",W[nvar]);
		fprintf(outfile,"\n%g seconds used for optimization\n",etime);
	}
	fprintf(outfile,"\n");
}/*print_thetab*/

/*set negative probabilities to zero and recalculate others in column
If too negative, report failure*/
boolean check_Pset()
{
	int i,j;
	double colsum,smallest;
	double ZERO=0.;
	smallest=ZERO;
	for(j=0;j<td->ntimes * td->s;j++){
		colsum=ZERO;
		for(i=0;i<td->s;i++){/*set any negative probabilities to zero*/
			if(Pset_cat[i][j]<ZERO){
				if(Pset_cat[i][j]<smallest) smallest=Pset_cat[i][j];
				Pset_cat[i][j]=ZERO;
			}
			colsum+=Pset_cat[i][j];
		}
		for(i=0;i<td->s;i++) Pset_cat[i][j]=Pset_cat[i][j]/colsum;
	}
	if (smallest<PROB_PROBLEM){
		fprintf(stderr,"transition prob %f, abandoning optimization\n",smallest);
		return FALSE;
	}
	else return TRUE;
}/*check_Pset*/

/*print out initial parameter estimates*/
void print_inittheta(double *theta, int nvar, int rep, FILE *outfile)
{
  int i,j;

  fprintf(outfile,"\nReplicate %d\n",rep);
  fprintf(outfile,"Starting conditions for optimization:\n");
  back_LVparms(theta,nvar);/*back-transform parameters*/
  fprintf(outfile,"Initial frequencies:\n");
  for(i=0;i<td->s;i++) fprintf(outfile,"%.16e ",thetab[i]);
  fprintf(outfile,"\n");	
  fprintf(outfile,"Rates:\n");
  for(i=0;i<td->s;i++){
    for(j=0;j<td->s;j++){
	if(i==j) fprintf(outfile,"%.16e ",0.);
	else fprintf(outfile,"%.16e ",get_aij(i,j,td->s));
    }
    fprintf(outfile,"\n");
  }
}/*print_inittheta*/
