/*
 *  Copyright (C) 2006 by Matthew Spencer
 *
 *  This program is free software; you may redistribute it and/or modify it
 *  under the terms of the GNU General Public License as published by the Free
 *  Software Foundation; either version 2 of the License, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful, but
 *  WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *  or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 *  For any other enquiries write to Matthew Spencer, School of Biological Sciences,
 *  University of Liverpool, Crown Street, Liverpool, L69 7ZB, UK.
 *
 *  Or send E-mail to m.spencer@liverpool.ac.uk
 */

#include <stdio.h>
#include <stdlib.h>


double **make_2darray(int m,int n);
double *make_1darray(int m);
void free_2darray(double **A,int m);
void print_2darray(double **A, int m, int n);


/*allocate memory for an mxn array of pointers to double*/
double **make_2darray(int m,int n)
{
	double **A;
	int i;

	A=(double **)calloc(m,sizeof(double*));
	if(A==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed in make_2darray");
		(void)exit(EXIT_FAILURE);
	}

	for(i=0;i<m;i++){
		A[i]=(double*)calloc(n,sizeof(double));
		if(A[i]==NULL){
			(void)fprintf(stderr,"ERROR: calloc failed in make_2darray");
			(void)exit(EXIT_FAILURE);
		}
	}
	return A;
}

/*allocate memory for an array of m doubles*/
double *make_1darray(int m)
{
	double *A;
	A=(double *)calloc(m,sizeof(double));
	if(A==NULL){
		(void)fprintf(stderr,"ERROR: calloc failed in make_1darray");
		(void)exit(EXIT_FAILURE);
	}
	return A;
}

/*free memory for mxn array of pointers to double*/
void free_2darray(double **A,int m)
{
	int i;
	for(i=0;i<m;i++){
		free(A[i]);
	}
	free(A);
}

/*print contents of 2d array*/
void print_2darray(double **A, int m, int n)
{
	int i,j;
	for(i=0;i<m;i++){
		for(j=0;j<n;j++){
			printf("%f ",A[i][j]);
		}
		printf("\n");
	}
}
